/*
 *  LANPERF - PC Tech Journal LAN Performance Test
 *  Copyright (c) 1988, Ziff Communications Company 
 *  Rewritten 11/22/1988 entirely in C
 *  Extended  01/31/1989 with maximum block size of 65534, changing version
 *  number.
 *
 *      by Ben Myers
 *      Spirit of Performance, Inc.
 *      73 Westcott Road
 *      Harvard, MA  01451
 *      (508) 456-3889
 *      MCI Mail 357-1400
 *
 *  The original used MASM routines for some of the functions.
 *  The maximum block size permitted is now 16384 bytes, limited by the
 *  small memory model of Microsoft C5.0.
 *  Except for timer functions which are specific to IBM PC BIOS,
 *  LANPERF is now portable to other operating environments.
 *  Let me know of successes in porting it, and I would appreciate having
 *  a copy of the modified source.  Thank you... Ben.
 *
 *  To build LANPERF.EXE:
 *    cl /AH LANPERF.C   (Compiled with Microsoft C 5.0)
 */

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <share.h>
#include <io.h>
#include <time.h>
/* MS/PC-DOS specific definitions and function */
#include <dos.h>
#define STARTTIME  begint = g_ticks()
#define STOPTIME   endt = g_ticks()
/*   duration in seconds * 18.2 ticks / second   */
#define ENDTIMEDELTA  (duration / 5) + (duration * 18)
long g_ticks()
/* Returns the current number of clock ticks @ 18.2 ticks/second */
  {
    long Current_Ticks;
    long far *BIOS_Clock;
    /* Set up pointer to double word containing clock ticks in BIOS */
    FP_SEG(BIOS_Clock) = 0;
    FP_OFF(BIOS_Clock) = 0x046C;
    _disable();                    /* No interrupts */
    Current_Ticks = *BIOS_Clock;   /* Get the current number of ticks */
    _enable();                     /* Allow interrupts again */
    return (Current_Ticks);
  }

/* end of MS/PC-DOS specific definitions and function */

#define MAXBLK      65534    /* maximum blksize value */
/* for OS that does not distinguish between stream and sequential I/O */
#define O_STREAM    0

/* information used by rtest() and wtest() routines */

int  master;                /* did we start the test? */
unsigned int blksize;       /* block I/O size in bytes */
unsigned int filesize;      /* size of file in blksize blocks */
char ioshare;               /* sharing mode for tests */
char iobuf[2*MAXBLK];       /* I/O buffer */
unsigned int duration;      /* length of test (seconds) */
long ops;                   /* total operations performed */
char fname[20];             /* name of test file created */

int fhandle;                /* file handle */
FILE *streamid;             /* stream I/O file */
int fileexists;             /* return indicator from open, != 0 exists */
unsigned int i;             /* loop counter */
long begint;  /* Start time in PC BIOS clock ticks */
long endt;    /* Stop time in PC BIOS clock ticks */
long CalcEndTime;

char *modename[] =
	{
	"Compatibility", "Deny read/write", "Deny write",
	"Deny read", "Deny none", "??"
	};

main(int argc, char **argv)
{
    unsigned int atou(const char *str);
    int i, err;
    int do_read, do_write;

    printf("LANPERF Version 1.1\n");
    printf("PC Tech Journal LAN Performance Test\n");
    printf("Copyright (c) 1988, Ziff Communications Company\n\n");

    if ( argc < 2 )
        {
        printf("Options:\n");
        printf(" t# - test length (seconds) (10-65535)\n");
	printf(" b# - block size (in bytes) (1-%u)\n", MAXBLK);
        printf(" f# - file size (in blocks) (1-65535)\n");
        printf(" w  - do sequential write test\n");
        printf(" r  - do sequential read test\n");
        printf(" dM - file sharing mode\n");
        printf("  ^ modes to deny (r w rw n c)\n");
        printf(" x  - use defaults:  t60 b512 f64 r w dc\n");
        exit(1);
        }

    /* set default values */
    blksize = 512;
    filesize = 64;
    duration = 60;
    ioshare = 0;    /* compatibility mode */
    do_read = 0;
    do_write = 0;

    /* decode arguments */
    for ( argv++, argc--; argc > 0; argv++, argc-- )
        {
        switch ( tolower(**argv) )
            {
            case 't' :
                duration = atou(*argv + 1);
                if ( duration < 10 )
                    {
                    printf("Illegal test time: %s\n", *argv);
                    exit(1);
                    }
                break;
            case 'b':
                blksize = atou(*argv + 1);
                if ( blksize == 0 || blksize > MAXBLK )
                    {
                    printf("Illegal block size: %s\n", *argv);
                    exit(1);
                    }
                break;
            case 'f':
                filesize = atou(*argv + 1);
                if ( filesize == 0 )
                    {
                    printf("Illegal file size: %s\n", *argv);
                    exit(1);
                    }
                break;
            case 'd':
                switch ( tolower(*(*argv + 1)) )
                    {
                    case 'r':
                        if ( tolower(*(*argv + 2)) == 'w' )
			    ioshare = SH_DENYRW; /* deny r/w */
                        else
			    ioshare = SH_DENYRD; /* deny read */
                        break;
                    case 'w':
			ioshare = SH_DENYWR;     /* deny write */
                        break;
                    case 'n':
			ioshare = SH_DENYNO;     /* deny none */
                        break;
                    case 'c':
			ioshare = SH_COMPAT;     /* compatibility */
                        break;
                    default:
                        printf("Illegal sharing mode: %s\n", *argv);
                        exit(1);
                    }
                break;
            case 'w':
                do_write = 1;
                break;
            case 'r':
                do_read = 1;
                break;
            case 'x':
                break;
            default:
                printf("unknown option: %s\n", *argv);
                exit(1);
            }
        }

    /* If read or write not specified, do both */
    if ( !do_read && !do_write )
        {
        do_read = 1;
        do_write = 1;
        }

    printf("Test time  : %u seconds\n", duration);
    printf("Block size : %u bytes\n", blksize);
    printf("File size  : %u blocks (%lu bytes)\n",
	 filesize, (long)filesize * (long)blksize);
    printf("Sharing    : %s\n\n", modename[ioshare >> 4]);
    printf("===> PRESS A KEY ON ANY STATION TO START <===\n\n");
    printf("           Total     Mean Response  Throughput\n");
    printf("Test     Operations    Time (ms)      (KB/s)\n");
    printf("----     ----------  -------------  ----------\n");

    /* Fill buffer with random data pattern */
    srand(19);
    for ( i = 0; i < sizeof(iobuf); i++ )
	iobuf[i] = rand ();

    strcpy(fname, "LANPERF.GO");
    unlink(fname);               /* in case it's already there */
    if ( (err = lpstart()) )
        fatal(err);

    if ( do_write )
        {
        printf("WRITE  :");
        if ( (err = wtest()) )
            fatal(err);
	printf("%8ld      %8.3f      %8.2f\n",
            ops,
            1000.0 * (float)duration / (float)ops,
            ((float)ops * (float)blksize) / (duration * 1024.0));
        }

    if ( do_read )
        {
        printf("READ   :");
        if ( (err = rtest()) )
            fatal(err);
        printf("%8ld      %8.3f      %8.2f\n",
            ops,
            1000.0 * (float)duration / (float)ops,
            ((float)ops * (float)blksize) / (duration * 1024.0));
        }

    if ( master )
        unlink("LANPERF.GO");

    exit(0);
}


fatal(int err)
{
static char *doscode[] =
    {
    "No error",
    "Invalid function",
    "File not found",
    "Path not found",
    "Too many open files",
    "Access denied",
    "Invalid handle",
    "Memory is trashed",
    "Insufficient memory",
    "Invalid memory block",
    "Bad environment",
    "Invalid format",
    "Invalid access code",
    "Invalid data",
    "??",
    "Invalid drive",
    "Can't remove current directory",
    "Not same device",
    "No more matching files",
    };
    
    if ( err < 0 )
        printf("\nERROR: possible disk full?");
    else
        {
        printf("\nDOS error %u, ", err);
        if ( err >= sizeof(doscode)/sizeof(doscode[0]) )
            printf("??\n");
        else
            printf("%s\n",doscode[err]);
        }
    unlink("LANPERF.GO");
    exit(1);
}

#include <ctype.h>
#include <limits.h>

unsigned int atou(const char *str)
{
    unsigned long tmp = 0;

    /* convert a string to an unsigned integer */
    while ( *str )
        {
        if ( isdigit(*str) )
            tmp = tmp * 10 + (*str++ - '0');
        else
            return(0);      /* garbage */
        }
    if ( tmp > UINT_MAX )
        return(0);          /* number too big */
    return((unsigned int)tmp);
}

int lpstart ()

{
int gotch, achar;
struct stat file_info;

gotch = 0;
fileexists = 0;
while ((gotch == 0) | (fileexists == 0))
  {
  gotch = kbhit();
  if ( gotch == 0 )
    {
    gotch = kbhit();
    if ( gotch == 0 )
      {
      gotch = kbhit();
      if ( gotch == 0 )
	fileexists = stat ( fname, &file_info );
      }
    }
  }
  if (gotch != 0)      /* a character in the buffer */
    {
    achar = getch();   /* get it */
    if (achar == 0)
      achar= getch();  /* extended character */
    }
  if (fileexists != 0)     /* file doesn't already exist */
    {
    fhandle =              /* create and close it */
      sopen ( fname, O_CREAT | O_STREAM | O_TRUNC | O_RDWR, ioshare,
		S_IREAD | S_IWRITE );
    if (fhandle != -1)
      fileexists = 0;
    else
      fileexists = -1;
    if ((close (fhandle)) != -1)
      fileexists =0;
    else
      fileexists = -1;
    }
  return(fileexists);
}

int rtest ()
{
register unsigned int i;
int nbytes, n_blks;

    fhandle =              /* create and initialize it */
      sopen ( fname, O_CREAT | O_STREAM | O_TRUNC | O_RDWR, ioshare,
		S_IREAD | S_IWRITE );
    fileexists = (fhandle != -1);
    setmode ( fhandle, O_BINARY );
    for (i = 0; i < filesize; i++)
      {
      /* vary position within buffer to defeat cacheing schemes */
      if ((nbytes = write ( fhandle, &iobuf[i % MAXBLK], blksize )) < blksize)
	{
	printf ( "rtest: error in initializing file." );
	close (fhandle);
	unlink (fname);
	exit (1);
	}
      }
    lseek (fhandle, 0L, 0);  /* back to beginning of file */

    fileexists = ((close (fhandle)) != -1);
    fhandle =              /* open to read now */
      sopen ( fname, O_STREAM | O_RDONLY, ioshare );
      setmode ( fhandle, O_BINARY );
      ops = 0;
      n_blks = filesize;
      /* Calculate end time in ticks, adding duration * 18.2 ticks / sec */
      CalcEndTime = (STARTTIME) + ENDTIMEDELTA;
      do
	{
	for (i = 0; i < filesize; i++)
	  {
	  read ( fhandle, iobuf, blksize );

	  ops++;
	  if (CalcEndTime <= (STOPTIME)) goto rtestexit;
	  }
	  /* after every pass through the file, seek back to beginning */
	  lseek (fhandle, 0L, 0);
	}
      while (CalcEndTime > endt);
      rtestexit:
      close (fhandle);
      unlink (fname);
      return (0);
}

int wtest ()
{
register unsigned int i;
int nbytes, n_blks;

    /* create and initialize file */

    fhandle =
      sopen ( fname, O_CREAT | O_STREAM | O_TRUNC | O_RDWR, ioshare,
		S_IREAD | S_IWRITE );
    setmode ( fhandle, O_BINARY );
      ops = 0;
      n_blks = filesize;
      /* Calculate end time in ticks, adding duration * 18.2 ticks / sec */
      CalcEndTime = (STARTTIME) + ENDTIMEDELTA;
      do
	{
	for (i = 0; i < filesize; i++)
	  {
	  /* vary position within buffer to defeat cacheing schemes */

	  write ( fhandle, &iobuf[i % MAXBLK], blksize );

	  ops++;
	  if (CalcEndTime <= (STOPTIME)) goto wtestexit;
	  }
	  /* after every pass through the file, seek back to beginning */
	  lseek (fhandle, 0L, 0);
	}
      while (CalcEndTime > endt);
      wtestexit:
      close (fhandle);
      unlink (fname);
      return (0);

}
